﻿Imports System.ComponentModel
Imports System.Windows.Forms

Namespace System.Data
   Public Enum BusyState : None : Filling : Saving : End Enum
   ''' <summary>some common basics of DatasetXmlAdapter and DatasetAdapter</summary>
   Public MustInherit Class DatasetAdapterBase

      Public Shared ReadOnly Attachs As New ComponentProperty(Of DataSet, DatasetAdapterBase)
      Public BusyState As BusyState = BusyState.None
      Protected _DataSet As DataSet
      Public HasChanges As Func(Of Boolean)
      Public ReadOnly HandleFormClosing As FormClosingEventHandler = AddressOf _HandleFormClosing
      Protected _DatasetAttach As DataTableAttach.DatasetAttach

      Protected MustOverride Sub _Save(atts As List(Of DataTableAttach))
      'Protected MustOverride Sub _Save(tbls As IEnumerable(Of DataTable))
      Protected MustOverride Sub _Fill(tbls As IEnumerable(Of DataTable))
      Public Sub Fill(tbls As Collections.Generic.IEnumerable(Of DataTable))
         BusyState = Data.BusyState.Filling
         Dim sw = Stopwatch.StartNew
         _Fill(tbls)
         Dbg("Filled in ", sw.ElapsedMilliseconds, "ms")
         BusyState = Data.BusyState.None
      End Sub

      Public Overridable Property DataSet() As DataSet
         Get
            Return _DataSet
         End Get
         Set(value As DataSet)
            If _DataSet Is value Then Return
            'If _DataSet.NotNull Then Attachs.Remove(_DataSet)
            _DataSet = value
            If _DataSet.Null Then Return
            'Attachs.Add(_DataSet, Me)
            HasChanges = AddressOf _HasChanges
            _DatasetAttach = _DataSet.Attach
            'RankedAttachs = _DataSet.GetRankedTables.Select(Function(tb) tb.Attach).ToArray
         End Set
      End Property

      Private Shared _QuoteChars As Char() = "[]".ToCharArray
      Private Shared Function UnQuote(s As String) As String
         Return If(s.Null, "", String.Concat(s.Split(_QuoteChars)).ToLower)
      End Function

      ''' <summary> If form is Null Gui-Validation and sound-feedback will be omitted </summary>
      Public Sub Save(form As Form, Optional checkForChanges As Boolean = True)
         Dim atts = _DatasetAttach.TableAttachs(Persistance.YesOrGuess).ToList
         Dim suspendableBindingSources = atts.SelectMany(Function(att) att.SuspendableBindingSources).ToList
         If form.NotNull Then
            For Each bs In suspendableBindingSources
               Try
                  bs.EndEdit()
               Catch ex As DataException
                  Dim rw = bs.At
                  If rw.RowState <> DataRowState.Detached Then Throw
                  Dim msg = ex.Message.And2("\n\nDie aktuelle ", rw.GetType.Name, " ist ungültig.", "\ndiese vom Speichern ausnehmen [OK], oder Speichern ganz abbrechen?")
                  If MessageBox.Show(msg, "invalid currentRow", MessageBoxButtons.OKCancel) = DialogResult.Cancel Then Return
                  bs.CancelEdit()
               End Try
            Next
            If Not form.Validate() Then
               System.Media.SystemSounds.Hand.Play()
               Return
            End If
            System.Media.SystemSounds.Asterisk.Play()
         End If
         If checkForChanges AndAlso Not HasChanges.Invoke() Then Return
         BusyState = Data.BusyState.Saving
         _Save(atts)
         For Each bs In suspendableBindingSources
            'Bug-Workaround: Bei untergeordneten BindingSources kann die Sort-Property von der zugrundeliegenden DataView-Sort-Property abweichen
            'wenn AcceptChanges die Dataviews restauriert nimmt es dann die falschen Sorts der BindingSources
            Dim dv1 = TryCast(bs.List, DataView)
            If dv1.NotNull AndAlso UnQuote(dv1.Sort) <> UnQuote(bs.Sort) Then bs.Sort = dv1.Sort
         Next
         _DataSet.AcceptChanges()
         BusyState = Data.BusyState.None
         With _DatasetAttach
            If ._SavedCallback.NotNull Then ._SavedCallback.Invoke()
         End With
      End Sub

      Private Function _HasChanges() As Boolean
         For Each att In _DatasetAttach.TableAttachs(Persistance.No)
            att.Table.AcceptChanges()
         Next
         Return _DataSet.HasChanges
      End Function

      Private Sub _HandleFormClosing(sender As Object, e As CancelEventArgs)
         If e.Cancel Then Return
         Dim frm = DirectCast(sender, Form)
         If Not HasChanges.Invoke() Then Return
            'Select Case MessageBox.Show(frm, "Änderungen speichern?", "Das Ende ist nahe", MessageBoxButtons.YesNoCancel)
            '   Case Windows.Forms.DialogResult.Yes
            Save(frm, False)
            '   Case Windows.Forms.DialogResult.Cancel
            '      e.Cancel = True
            '      Return
            'End Select
            e.Cancel = False
      End Sub

   End Class

End Namespace
