﻿#Region "FileHeader"
#If False Then
DgvCellStyler(Of T As DataRow) wrappert 3 Dgv-Events: CellPainting, CellFormatting, CellParsing. Und reichert die EventArgs mit ZusatzInfos an
Die ZusatzInfo besteht in Angabe der betroffenen DgvColumn (statt nur des ColumnIndex'), sowie der betroffenen gebundenen typisierten DataRow.
Auch haben die Eventargs Ausführungs-Logik, und setzen den Handled-/Applied-Member des gewrapperten EventArgs dann selbsttätig true, wenn die Logik betätigt wurde.
#End If '-- Options, Imports
#End Region 'FileHeader

Namespace Global.System.Windows.Forms

	Public Class DgvCellStyler(Of T As DataRow) : Inherits EventArgs
		Public Event Painting(e As DgvCellStyler(Of T))
		Public Event Formatting(e As DgvCellStyler(Of T))
		Public Event Parsing(e As DgvCellStyler(Of T))

		Public FormattingEArgs As DataGridViewCellFormattingEventArgs
		Public PaintingEArgs As DataGridViewCellPaintingEventArgs
		Public ParsingEArgs As DataGridViewCellParsingEventArgs


		Private _ColumnIndexFilter As Integer()
		Public Cell As DataGridViewCell
		Public Graphics As Graphics
		Public ReadOnly Brush As New SolidBrush(Color.AliceBlue)
		Public WithEvents Dgv As DataGridView

		Public Function Row() As T
			Return DirectCast(DirectCast(Cell.OwningRow.DataBoundItem, DataRowView).Row, T)
		End Function
		Public Function DgvColumn() As DataGridViewColumn
			Return Cell.OwningColumn
		End Function

		Public Sub SetObservedColumns(ParamArray columns As DataGridViewColumn())
			_ColumnIndexFilter = columns.ToArray(Function(x) x.Index)
			Me.Dgv = columns(0).DataGridView
		End Sub

		Public Sub SetGrid(dgv As DataGridView)
			Me.Dgv = dgv
		End Sub

		Public Sub PaintBackground(Optional color As Color = Nothing, Optional selectionBackColor As Color = Nothing)
			With PaintingEArgs
				Dim selected = 0 <> (.State And DataGridViewElementStates.Selected)
				If selected Then color = selectionBackColor
				If color = Drawing.Color.Empty Then
					.PaintBackground(.CellBounds, selected)
				Else
					Brush.Color = color
					Graphics.FillRectangle(Brush, .CellBounds)
				End If
				.Handled = True
			End With
		End Sub

		Public Sub PaintContent()
			With PaintingEArgs
				.PaintContent(.CellBounds)
				.Handled = True
			End With
		End Sub
		Public Property Value As Object
			Get
				Return If(FormattingEArgs IsNot Nothing, FormattingEArgs.Value, ParsingEArgs.Value)
			End Get
			Set(value As Object)
				If FormattingEArgs IsNot Nothing Then
					FormattingEArgs.Value = value
					FormattingEArgs.FormattingApplied = True
				Else
					ParsingEArgs.Value = value
					ParsingEArgs.ParsingApplied = True
				End If
			End Set
		End Property

		Public Sub SetHandledExplicit(value As Boolean)
			If FormattingEArgs Is Nothing Then PaintingEArgs.Handled = value Else FormattingEArgs.FormattingApplied = value
		End Sub
		Public ReadOnly Property CellBounds As Rectangle
			Get
				Return PaintingEArgs.CellBounds
			End Get
		End Property
		Public ReadOnly Property Selected As Boolean
			Get
				Return 0 <> (PaintingEArgs.State And DataGridViewElementStates.Selected)
			End Get
		End Property

		Private Function InitData(columnIndex As Integer, rowIndex As Integer) As Boolean
			If _ColumnIndexFilter.NotNull AndAlso Not _ColumnIndexFilter.Contains(columnIndex) Then Return False
			If rowIndex > Dgv.RowCount - If(Dgv.AllowUserToAddRows, 2, 1) Then Return False
			FormattingEArgs = Nothing
			PaintingEArgs = Nothing
			ParsingEArgs = Nothing
			Cell = Dgv(columnIndex, rowIndex)
			Return True
		End Function

		Private Sub Dgv_CellPainting(sender As Object, e As DataGridViewCellPaintingEventArgs) Handles Dgv.CellPainting
			If e.ColumnIndex < 0 OrElse e.RowIndex < 0 Then Return
			If Not InitData(e.ColumnIndex, e.RowIndex) Then Return
			PaintingEArgs = e
			Graphics = e.Graphics
			RaiseEvent Painting(Me)
		End Sub

		Private Sub Dgv_CellFormatting(sender As Object, e As DataGridViewCellFormattingEventArgs) Handles Dgv.CellFormatting
			If Not InitData(e.ColumnIndex, e.RowIndex) Then Return
			If Dgv.IsCurrentCellInEditMode AndAlso Cell Is Dgv.CurrentCell Then Return
			FormattingEArgs = e
			RaiseEvent Formatting(Me)
		End Sub

		Private Sub Dgv_CellParsing(sender As Object, e As DataGridViewCellParsingEventArgs) Handles Dgv.CellParsing
			If Not InitData(e.ColumnIndex, e.RowIndex) Then Return
			ParsingEArgs = e
			RaiseEvent Parsing(Me)
		End Sub

	End Class

End Namespace
